# -*- coding: utf-8 -*-
# Generated by Django 1.9.2 on 2016-02-20 18:28
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('pushy', '0003_auto_20150902_2001'),
    ]

    operations = [
        migrations.AlterField(
            model_name='pushnotification',
            name='sent',
            field=models.SmallIntegerField(choices=[(0, 'Not Sent'), (2, 'In Progress'), (1, 'Sent')], default=0),
        ),
    ]
